// *********************************************************************
// *  This software uses the Perseus API which is defined through the  *
// *  following four files of the Microtelecom Software Defined Radio  *
// *  Developer Kit (SDRK):                                            *
// *  main.cpp, perseusdll.cpp, perseusdefs.h, perseusdll.h            *
// *  To use the Perseus hardware you need the perseususb.dll file     *
// *  You also need at least one of the following files:               *
// *  perseus125k24v21.sbs                                             *
// *  perseus250k24v21.sbs                                             *
// *  perseus500k24v21.sbs                                             *
// *  perseus1m24v21.sbs                                               *
// *  perseus2m24v21.sbs                                               *
// *  The dll and the sbs files are copyright protected and only       *
// *  available at the official Microtelecom Internet site:            *
// *  http://www.microtelecom.it                                       *
// *********************************************************************

#include <string.h>
#include "globdef.h"
#include "uidef.h"
#include "vernr.h"
#include "perseusdef.h"
#include "rusage.h"
#include "thrdef.h"
#include "sdrdef.h"
#include "fft1def.h"
#include "screendef.h"


#if(OSNUM == OSNUM_LINUX)
#include <unistd.h>
#endif

void open_perseus(void);
char *perseus_name(void);
char *perseus_frname(void);
int perseus_eeprom_read(unsigned char *buf, int addr, unsigned char count);
void perseus_store_att(unsigned char ch);
void perseus_store_presel(unsigned char ch);
void lir_perseus_read(void);
void start_perseus_read(void);
void perseus_stop(void);
int perseus_load_proprietary_code(int rate_no);
void set_perseus_frequency(void);

char *perseus_parfil_name="par_perseus";

P_PERSEUS pers;

char *perseus_parm_text[MAX_PERSEUS_PARM]={"Rate no",           //0
                                           "Sampl. clk adjust", //1
                                           "Preselector",       //2
                                           "Preamp",            //3
                                           "Dither",            //4
                                           "Check"};            //5

#if(OSNUM == OSNUM_WINDOWS)

double rate_factor[MAX_PERSEUS_RATES]={1666.6666667,  //48kHz
                                       840.0,         //95.238 kHz
                                       832.0,         //96.154 kHz
                                       640.0,         //125 kHz
                                       416.0,         //192.308 kHz
                                       320.0,         //250 kHz
                                       160.0,         //500 kHz
                                       80.0,          //1 MHz
                                       40.0};         //2 MHz
#endif

#if(OSNUM == OSNUM_LINUX)
double rate_factor[PERSEUS_RATES_BUFSIZE];
#endif


#if(OSNUM == OSNUM_WINDOWS)
char *perseus_bitstream_names[MAX_PERSEUS_RATES]={
                 "perseus48k24v31",
                 "perseus95k24v31",
                 "perseus96k24v31",
                 "perseus125k24v21",
                 "perseus192k24v31",
                 "perseus250k24v21",
                 "perseus500k24v21",
                 "perseus1m24v21",
                 "perseus2m24v21"};
#endif

#if(OSNUM == OSNUM_LINUX)
char *perseus_bitstream_names[MAX_PERSEUS_RATES]={
                 "perseus95k24v31.rbs",
                 "perseus125k24v21.rbs",
                 "perseus250k24v21.rbs",
                 "perseus500k24v21.rbs",
                 "perseus1m24v21.rbs",
                 "perseus2m24v21.rbs"};
#endif

float perseus_filter_cutoff[MAX_PERSEUS_FILTER+1]={
                  1.7,
                  2.1,
                  3.0,
                  4.2,
                  6.0,
                  8.4,
                 12.0,
                 17.0,
                 24.0,
                 32.0,
                 BIG};

// Data structure which holds controls for the receiver ADC and DDC

// Perseus EEPROM Definitions and data structures
#define ADDR_EEPROM_PRODID 8
#pragma pack(1)
typedef struct {
unsigned short int sn;       // Receiver Serial Number
unsigned short int prodcode; // Microtelecom Product Code
unsigned char hwrel;         // Product release
unsigned char hwver;         // Product version
unsigned char signature[6];  // Microtelecom Original Product Signature
} eeprom_prodid;
#pragma pack()

#if(OSNUM == OSNUM_WINDOWS)
char *rate_names[MAX_PERSEUS_RATES]={"48k",
                                     "95.2k",
                                     "96k",
                                     "125k",
                                     "192k",
                                     "250k",
                                     "500k",
                                     "1M",
                                     "2M"};
#endif



int display_perseus_parm_info(int *line)
{
char s[80];
int i, errcod;
errcod=read_sdrpar(perseus_parfil_name, MAX_PERSEUS_PARM, 
                                     perseus_parm_text, (int*)((void*)&pers));
if(errcod == 0)
  {
#if(OSNUM == OSNUM_LINUX)
    fill_perseus_rates();
    for(i=0; i<no_of_perseus_rates; i++)
      {
      rate_factor[i]=PERSEUS_SAMPLING_CLOCK/perseus_rates[i];
      }
#endif
#if(OSNUM == OSNUM_WINDOWS)
    for(i=0; i<MAX_PERSEUS_RATES; i++)
      {
      perseus_rates[i]=PERSEUS_SAMPLING_CLOCK/rate_factor[i];
      }
    no_of_perseus_rates=MAX_PERSEUS_RATES;
#endif
  settextcolor(7);
  sprintf(s,"rate number        = %i ", pers.rate_no);
  if(perseus_rates[pers.rate_no] > 999999.9)
    {
    sprintf(s,"rate number        = %i (%8.3f MHz)", 
                         pers.rate_no, 0.000001*perseus_rates[pers.rate_no]);
    }
  else
    {
    sprintf(s,"rate number        = %i (%8.3f kHz)", 
                         pers.rate_no, 0.001*perseus_rates[pers.rate_no]);
    }
  lir_text(24,line[0],s);
  SNDLOG"\n%s",s);
  line[0]+=1;
  sprintf(s,"clock adjustment   = %i", pers.clock_adjust);
  lir_text(24,line[0],s);
  SNDLOG"\n%s",s);
  line[0]+=1;
  sprintf(s,"preselector        = %i", pers.presel);
  lir_text(24,line[0],s);
  SNDLOG"\n%s",s);
  line[0]+=1;
  sprintf(s,"preamplifier       = %i", pers.preamp);
  lir_text(24,line[0],s);
  SNDLOG"\n%s",s);
  line[0]+=1;
  sprintf(s,"dither             = %i", pers.dither);
  lir_text(24,line[0],s);
  SNDLOG"\n%s\n",s);
  }
return (errcod);
}

void init_perseus(void)
{
int i, k;
char s[80];
char *ss;
int *sdr_pi;
int line;
char *looking;
eeprom_prodid prodid;
FILE *perseus_file;
looking="Looking for a Perseus on the USB port.";
settextcolor(12);
lir_text(10,10,looking);
SNDLOG"\n%s",looking);
lir_text(10,11,"Reset CPU, USB and SDR hardware if system hangs here.");
lir_refresh_screen();
settextcolor(14);
open_perseus();
settextcolor(7);
clear_lines(10,11);
lir_refresh_screen();
line=0;
if(sdr != -1)
  {
  lir_text(5,5,"A Perseus is detected on the USB port.");
  lir_text(5,6,"Do you want to use it for RX input (Y/N)?");
qpers:;
  await_processed_keyboard();
  if(kill_all_flag) goto pers_errexit;
  if(lir_inkey == 'Y')
    {
#if(OSNUM == OSNUM_WINDOWS)
    ss=perseus_frname();
    if(ss == NULL)goto pers_errexit;
    sprintf(s,"USB Friendly device Name is: %s",ss);
    lir_text(2,8,s);
    SNDLOG"%s\n",s);
    ss=perseus_name();
    if(ss == NULL)goto pers_errexit;
    sprintf(s,"USB Device Name is: %s",ss);
    lir_text(2,9,s);
    SNDLOG"%s\n",s);
#else
    ss="Perseus";
#endif
    ui.rx_addev_no=PERSEUS_DEVICE_CODE;
    ui.rx_input_mode=IQ_DATA+DIGITAL_IQ+DWORD_INPUT;
    ui.rx_rf_channels=1;
    ui.rx_ad_channels=2;
    ui.rx_admode=0;
    perseus_eeprom_read((unsigned char*)&prodid,
                                       ADDR_EEPROM_PRODID, sizeof(prodid));
    if(kill_all_flag)goto pers_errexit;
    sprintf(s,"Microtelecom product code: 0x%04X", prodid.prodcode);
    lir_text(2,10,s);
    SNDLOG"%s\n",s);
    sprintf(s,"Serial Number: %05hd-%02hX%02hX-%02hX%02hX-%02hX%02hX",
				(unsigned short)prodid.sn,
				(unsigned short)prodid.signature[5],
				(unsigned short)prodid.signature[4],
				(unsigned short)prodid.signature[3],
				(unsigned short)prodid.signature[2],
				(unsigned short)prodid.signature[1],
				(unsigned short)prodid.signature[0]);
    lir_text(2,11,s);
    SNDLOG"%s\n",s);
    sprintf(s,"Hardware version/revision: %d.%d",
                         (unsigned)prodid.hwver, (unsigned)prodid.hwrel);
    lir_text(2,12,s);
    SNDLOG"%s\n",s);
    lir_text(10,16,"PRESS ANY KEY");
    await_processed_keyboard();
    if(kill_all_flag) goto pers_errexit;
    clear_screen();
    sprintf(s,"%s selected for input",ss);
    lir_text(10,line,s);
    line+=2;
#if(OSNUM == OSNUM_WINDOWS)
    for(i=0; i<MAX_PERSEUS_RATES; i++)
      {
      perseus_rates[i]=PERSEUS_SAMPLING_CLOCK/rate_factor[i];
      }
    no_of_perseus_rates=MAX_PERSEUS_RATES;
#else     
    fill_perseus_rates();
    for(i=0; i<no_of_perseus_rates; i++)
      {
      rate_factor[i]=PERSEUS_SAMPLING_CLOCK/perseus_rates[i];
      }
#endif    
    k=no_of_perseus_rates-1;
    if(ui.operator_skil == OPERATOR_SKIL_NEWCOMER)k-=2;
    for(i=0; i<no_of_perseus_rates; i++)
      {
      if(i>k)settextcolor(8);
      if(perseus_rates[i] > 999999.9)
        {
        sprintf(s,"%d %8.3f MHz ",i,0.000001*perseus_rates[i]);
        }
      else
        {
        sprintf(s,"%d %8.3f kHz ",i,0.001*perseus_rates[i]);
        }
      lir_text(5,line,s);
      line++;
      }
    settextcolor(7);
    line++;
    sprintf(s,"Set USB sampling rate (by number 0 to %d)",k);
    lir_text(5,line,s);
    pers.rate_no=lir_get_integer(50, line, 2, 0, k);
    if(kill_all_flag)goto pers_errexit;
    line++;
    if(ui.operator_skil == OPERATOR_SKIL_NEWCOMER)
      {
      pers.clock_adjust=0;
      }
    else
      {
      sprintf(s,"Set sampling clock shift (Hz))");
      lir_text(5,line,s);
      pers.clock_adjust=lir_get_integer(50, line, 5,-10000,10000);
      if(kill_all_flag)goto pers_errexit;
      line++;
      }
    adjusted_sdr_clock=PERSEUS_SAMPLING_CLOCK+pers.clock_adjust;
    ui.rx_ad_speed=adjusted_sdr_clock/rate_factor[pers.rate_no];
    if(ui.operator_skil == OPERATOR_SKIL_NEWCOMER)
      {
      pers.presel=1;
      pers.preamp=1;
      pers.dither=1;
      }
    else
      {
      lir_text(5,line,"Enable preselector ? (0/1)");
      pers.presel=lir_get_integer(32, line, 1, 0, 1);
      if(kill_all_flag)goto pers_errexit;
      line++;
      lir_text(5,line,"Enable preamp ? (0/1)");
      pers.preamp=lir_get_integer(27, line, 1, 0, 1);
      if(kill_all_flag)goto pers_errexit;
      line++;
      lir_text(5,line,"Enable dither ? (0/1)");
      pers.dither=lir_get_integer(27, line, 1, 0, 1);
      if(kill_all_flag)goto pers_errexit;
      line++;
      }
    pers.check=PERSEUS_PAR_VERNR;
    perseus_file=fopen(perseus_parfil_name,"w");
    if(perseus_file == NULL)
      {
pers_errexit:;
      close_perseus();
      clear_screen();
      ui.rx_addev_no=UNDEFINED_DEVICE_CODE;
      return;
      }
    sdr_pi=(int*)(&pers);
    for(i=0; i<MAX_PERSEUS_PARM; i++)
      {
      fprintf(perseus_file,"%s [%d]\n",perseus_parm_text[i],sdr_pi[i]);
      }
    parfile_end(perseus_file);
    }
  else
    {
    if(lir_inkey != 'N')goto qpers;
    clear_screen();
    }
  close_perseus();
  }
else
  {
  SNDLOG"\nopen_perseus failed.\n");
  }
return;
}

void set_perseus_att(void)
{
int i;
i=-fg.gain/10;
perseus_store_att(i);
}

void perseus_input(void)
{
#if RUSAGE_OLD == TRUE
int local_workload_counter;
#endif
int rxin_local_workload_reset;
int i, j, errcod;
char s[80];
double dt1, read_start_time, total_reads;
int timing_loop_counter,timing_loop_counter_max,initial_skip_flag;
float t1;
int local_att_counter;
int local_nco_counter;
#if OSNUM == OSNUM_LINUX
clear_thread_times(THREAD_PERSEUS_INPUT);
#endif
#if RUSAGE_OLD == TRUE
local_workload_counter=workload_counter;
#endif
local_att_counter=sdr_att_counter;
local_nco_counter=sdr_nco_counter;
dt1=current_time();
j=0;
while(sdr == -1)
  {
  open_perseus();
  if(sdr == -1)
    {
    sprintf(s,"Waiting %.2f", current_time()-dt1);
    lir_text(3,5,s);
    lir_refresh_screen();
    if(kill_all_flag)goto perseus_init_error_exit;
    lir_sleep(100000);
    if(j==0)
      {
      clear_screen();
      j=1;
      }
    }
  }
// ****************************************************
// We have a connection to the Perseus.
i=read_sdrpar(perseus_parfil_name, MAX_PERSEUS_PARM, 
                                     perseus_parm_text, (int*)((void*)&pers));
errcod=1080;
if(i != 0)goto perseus_error_exit;
if( pers.check != PERSEUS_PAR_VERNR ||
    pers.rate_no < 0 ||
    abs(pers.clock_adjust) > 10000)goto perseus_error_exit;
#if(OSNUM == OSNUM_WINDOWS)
if(pers.rate_no  >= MAX_PERSEUS_RATES)goto perseus_error_exit;
#endif
#if(OSNUM == OSNUM_LINUX)
fill_perseus_rates();
if(pers.rate_no  >= no_of_perseus_rates)goto perseus_error_exit;
for(i=0; i<no_of_perseus_rates; i++)
  {
  rate_factor[i]=PERSEUS_SAMPLING_CLOCK/perseus_rates[i];
  }
#endif    
errcod=0;
adjusted_sdr_clock=PERSEUS_SAMPLING_CLOCK+pers.clock_adjust;
t1=ui.rx_ad_speed-adjusted_sdr_clock/rate_factor[pers.rate_no];
if(fabs(t1/ui.rx_ad_speed) > 0.0001)
  {
  errcod=1091;
  goto perseus_error_exit;
  }
errcod=perseus_load_proprietary_code(pers.rate_no);
if(errcod != 0)goto perseus_error_exit;
ui.rx_ad_speed=adjusted_sdr_clock/rate_factor[pers.rate_no];
set_perseus_att();
set_perseus_frequency();
fft1_block_timing();
fft1_hz_per_point=(float)ui.rx_ad_speed/fft1_size;
if(thread_command_flag[THREAD_SCREEN]!=THRFLAG_NOT_ACTIVE)
  {
  while(thread_status_flag[THREAD_SCREEN]!=THRFLAG_ACTIVE &&
        thread_status_flag[THREAD_SCREEN]!=THRFLAG_IDLE &&
        thread_status_flag[THREAD_SCREEN]!=THRFLAG_SEM_WAIT)
    {
    if(thread_command_flag[THREAD_PERSEUS_INPUT] ==
                                           THRFLAG_KILL)goto perseus_error_exit;
    lir_sleep(10000);
    }
  }
start_perseus_read();
if(sdr != 0) goto perseus_error_exit;
#include "timing_setup.c"
thread_status_flag[THREAD_PERSEUS_INPUT]=THRFLAG_ACTIVE;
while(thread_command_flag[THREAD_PERSEUS_INPUT] == THRFLAG_ACTIVE)
  {
#if RUSAGE_OLD == TRUE
  if(local_workload_counter != workload_counter)
    {
    local_workload_counter=workload_counter;
    make_thread_times(THREAD_PERSEUS_INPUT);
    }
#endif
  if(local_att_counter != sdr_att_counter)
    {
    local_att_counter=sdr_att_counter;
    set_perseus_att();
    }
  if(local_nco_counter != sdr_nco_counter)
    {
    local_nco_counter=sdr_nco_counter;
    set_perseus_frequency();
    }
  lir_perseus_read();
  if(kill_all_flag)goto perseus_exit;
  while (!kill_all_flag && 
      ((timf1p_sdr-timf1p_pa+timf1_bytes)&timf1_bytemask) 
                                         >= (int)snd[RXAD].block_bytes)
    {
#include "input_speed.c"
    finish_rx_read();
    }
  }
perseus_exit:;
perseus_stop();
perseus_error_exit:;
if(errcod != 0)lirerr(errcod);
close_perseus();
perseus_init_error_exit:;
thread_status_flag[THREAD_PERSEUS_INPUT]=THRFLAG_RETURNED;
while(thread_command_flag[THREAD_PERSEUS_INPUT] != THRFLAG_NOT_ACTIVE)
  {
  lir_sleep(1000);
  }
}

